/**
 * Function for handling management protocol test functions
 * 
 * This file is part of DCC Monitor.
 *
 * Copyright 2007, 2008 Peter Lebbing <peter@digitalbrains.com>
 *
 * DCC Monitor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * DCC Monitor is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with DCC Monitor.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdint.h>
#include <stddef.h>
#include <stdio.h>
#include "../global.h"
#include "../display.h"
#include "test_emul.h"
#include "test_comm_proto_glue.h"
#include "test_comm_forward_handler.h"

/**
 * Identifiers for the type of test, for storage in test_data_item->test_num
 */
#define COMM_TEST_ID 0
#define FORWARD_TEST_ID 1

/**
 * Handle management protocol test functions frames
 *
 * This function is called by parse_manag when it encounters a "test functions"
 * frame.
 *
 * Parameter start points to subdata for test functions.
 */
void test_dispatch (struct s_frame *frame, size_t start) {
  uint8_t testproto;
  struct test_data_item *test_data;
  int result;

  switch (frame->data[start]) {
    case 0:
      /* 
       * "Communication protocol" test
       * Frame subdata format: 0x00 PP
       * where PP == 0: end test
       *          != 0: use proto PP for test data
       */
      
      if (start + 2 != frame->len) {
        // Too much data
        printf (": Incorrect \"test\" management frame!\n");
        dump_hex_data (frame);
        return;
      }

      if (frame->data[start+1] != 0) {
        // Start test
        
        testproto = frame->data[start+1];
        if (testproto >= MAX_PROTOS) {
          // No such protocol number
          printf (": Incorrect \"test\" management frame!\n");
          dump_hex_data (frame);
          return;
        }

        // Check that there isn't a test already running
        if (test_emul_get_data (frame->addr) != NULL) {
          // There's a test already running
          printf (": \"Test\" management frame requests a test to be run, but a test is already running!\n");
          dump_hex_data (frame);
          return;
        }

        printf (": Starting \"Communication protocol\" test\n");
        // Initialise test
        test_emul_start (frame->addr, COMM_TEST_ID);
        init_comm_test (frame->addr, testproto);

        if (verbosity & VERB_MANAG) {
          dump_hex_data (frame);
        }
        return;
      }

      // Stop test
      test_data = test_emul_get_data (frame->addr);

      if (test_data == NULL || test_data->test_id != COMM_TEST_ID) {
        // No test to stop, error
        printf (": \"Communication protocol\" test stopped but never started!\n");
        dump_hex_data (frame);
        return;
      }

      // End the test and analyze result
      result = end_comm_test (frame->addr);
      if (result == 0) {
        // Okay
        printf (": \"Communication protocol\" test completed succesfully.\n");

      } else if (result == -1) {
        // Error in one of the data frames
        printf (": Bad frame(s) during \"Communication protocol\" test!\n");
      
      } else { // result == -2
        // Test ended prematurely (would have expected more frames)
        printf (": \"Communication protocol\" test ended prematurely!\n");
      }

      // Destroy test environment
      test_emul_end (frame->addr);

      if (verbosity & VERB_MANAG) {
        dump_hex_data (frame);
      }
      return;

    case 1:
      /* 
       * "Communication forward" test
       * Frame subdata format: 0x01 PP
       * where PP == 0: end test
       *          != 0: use proto PP for test data
       */
      
      if (start + 2 != frame->len) {
        // Too much data
        printf (": Incorrect \"test\" management frame!\n");
        dump_hex_data (frame);
        return;
      }

      if (frame->data[start+1] != 0) {
        // Start test
        
        testproto = frame->data[start+1];
        if (testproto >= MAX_PROTOS) {
          // No such protocol number
          printf (": Incorrect \"test\" management frame!\n");
          dump_hex_data (frame);
          return;
        }

        // Check that there isn't a test already running
        if (test_emul_get_data (frame->addr) != NULL) {
          // There's a test already running
          printf (": \"Test\" management frame requests a test to be run, but a test is already running!\n");
          dump_hex_data (frame);
          return;
        }

        if (frame->addr == 0) {
          printf (": \"Communication forward\" test should not be run on board with address 0!\n");
          printf ("\tYou will get a bogus frame from address 7.\n");

          // Yet we still open an environment to give a correct report when we
          // receive the "end test" frame
          test_emul_start (frame->addr, FORWARD_TEST_ID);

          if (verbosity & VERB_MANAG) {
            dump_hex_data (frame);
          }

          return;
        }

        printf (": Starting \"Communication forward\" test\n");
        // Initialise test
        test_emul_start (frame->addr, FORWARD_TEST_ID);
        init_forward_test (frame->addr, testproto);

        if (verbosity & VERB_MANAG) {
          dump_hex_data (frame);
        }
        return;
      }

      // Stop test
      test_data = test_emul_get_data (frame->addr);

      if (test_data == NULL || test_data->test_id != FORWARD_TEST_ID) {
        // No test to stop, error
        printf (": \"Communication forward\" test stopped but never started!\n");
        dump_hex_data (frame);
        return;
      }

      if (frame->addr == 0) {
        printf (": Bogus \"Communication forward\" test ended (no result).\n");

      } else {
        // End the test and analyze result
        result = end_forward_test (frame->addr);
        if (result == 0) {
          // Okay
          printf (": \"Communication forward\" test completed succesfully.\n");
        } else { // result == -2
          // Test ended prematurely (would have expected more frames)
          printf (": \"Communication forward\" test failed!\n");
        }
      }

      // Destroy test environment
      test_emul_end (frame->addr);

      if (verbosity & VERB_MANAG) {
        dump_hex_data (frame);
      }
      return;

    default:
      // Unknown test
      printf (": Incorrect \"test\" management frame!\n");
      dump_hex_data (frame);
      return;
  }
}
